--
-- DAFGearbox
-- 16/06/09
-- manual, automatic downspeed, full automatic powershift

DAFGearbox = {};

function DAFGearbox.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function DAFGearbox:load(xmlFile)
    self.numGear = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#count"), 0);
	self.gears = {};
    for i=1, self.numGear do
        local geari = string.format("vehicle.gears.gear%d", i);
        self.gears[i] = getXMLFloat(xmlFile, geari .. "#value");
    end;
	self.miniGear = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#min"), 1);
    self.down = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#down"), 5)/10;
	self.up = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#up"), 7)/10;
	self.realMinRpm = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#realMinRpm"), 800);
	self.realMaxRpm = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.gears#realMaxRpm"), 2600);
	self.gearNum = self.miniGear;
	self.gearMem = self.numGear;
	self.mode = 0;
	self.speedup = false;
	self.speeddown = false;
end;

function DAFGearbox:delete()
end;

function DAFGearbox:mouseEvent(posX, posY, isDown, isUp, button)
end;

function DAFGearbox:keyEvent(unicode, sym, modifier, isDown)

    if self:getIsActiveForInput() and self.isEntered then
	    if InputBinding.isPressed(InputBinding.SPEEDUP) then
			if self.mode == 2 then
				if self.gearMem < self.numGear then
					self.gearMem = self.gearMem + 1;
				end;
			else
				if self.gearNum < self.numGear then
					self.gearNum = self.gearNum + 1;
					self.gearMem = self.gearNum;
				end;
			end;
		elseif InputBinding.isPressed(InputBinding.SPEEDDOWN) then
			if self.mode == 2 then
				if self.gearMem > 1 then
					self.gearMem = self.gearMem - 1;
					self.gearNum = math.min(self.gearMem,self.gearNum);
				end;
			else
				if self.gearNum > 1 then
					self.gearNum = self.gearNum - 1;
					self.gearMem = self.gearNum;
				end;
			end;
		end;
		if InputBinding.isPressed(InputBinding.MODE) then
			if self.mode < 2 then
				self.mode = self.mode + 1;
			else
				self.mode = 0;
			end;
		end;
	end;
end;

function DAFGearbox:update(dt)
	if self:getIsActiveForInput() and self.isEntered then
		if  self.gearNum > self.miniGear and self.mode >= 1 and self.lastRoundPerMinute < (self.gears[self.gearNum-1]*self.down) then
			self.gearNum = self.gearNum - 1;
		elseif self.gearMem > self.gearNum and self.mode == 2 and self.lastRoundPerMinute > (self.gears[self.gearNum]*self.up)then
			self.gearNum = self.gearNum + 1;
		end;
	end;
	self.motor.maxRpm[3] = self.gears[self.gearNum];
end;

function DAFGearbox:draw()
	if self.isMotorStarted and self.isEntered then
		if self.motor.speedLevel == 0 then
			self.rpm = math.min(math.max(math.floor(self.lastRoundPerMinute*self.realMaxRpm / (self.gears[self.gearNum]*10))*10, self.realMinRpm),self.realMaxRpm);
		else
			self.rpm = math.min(math.max(math.floor(self.lastRoundPerMinute*self.realMaxRpm / (self.motor.maxRpm[motor.speedLevel]*10))*10, self.realMinRpm),self.realMaxRpm);
		end;
		--g_currentMission:addExtraPrintText("Rpm  // Rpm max               : "..string.format("%d ",self.lastRoundPerMinute).."/"..string.format("%d ",self.gears[self.gearNum]));
		setTextBold(true);
		renderText(0.83, 0.18, 0.05, string.format("%4d ",self.rpm).."rpm");
	end;
	if self.motor.speedLevel == 0 or self.motor.speedLevel == 3 then
		local text;
		if self.mode == 0 then
			g_currentMission:addExtraPrintText(string.format("Press %s to switch mode: %s",InputBinding.getButtonKeyName(InputBinding.MODE),"Manual Mode"));
			text = "Speed Up/Speed Down: "..string.format("%d ",self.gearNum).."/"..string.format("%d ",self.numGear);		
		elseif self.mode == 1 then
			g_currentMission:addExtraPrintText(string.format("Press %s to switch mode: %s",InputBinding.getButtonKeyName(InputBinding.MODE),"Automatic Downspeed"));
			text = "Speed Up/Speed Down: "..string.format("%d ",self.gearNum).."/"..string.format("%d ",self.numGear);
		else
			text = "Speed Up/Speed Down: "..string.format("%d ",self.gearNum).."/"..string.format("%d ",self.gearMem).."/"..string.format("%d ",self.numGear)
			g_currentMission:addExtraPrintText(string.format("Press %s to switch mode: %s",InputBinding.getButtonKeyName(InputBinding.MODE),"Full Automatic"));
		end;
		g_currentMission:addExtraPrintText(string.format("Press %s / %s : %s", InputBinding.getButtonKeyName(InputBinding.SPEEDUP), InputBinding.getButtonKeyName(InputBinding.SPEEDDOWN),text));
	end;
end;
